package java.security;

import java.util.Enumeration;
import sun.security.jca.GetInstance;
import sun.security.provider.PolicyFile;
import sun.security.util.SecurityConstants;
import java.util.WeakHashMap;
import sun.security.util.Debug;
import java.util.concurrent.atomic.AtomicReference;

public abstract class Policy
{
    public static final PermissionCollection UNSUPPORTED_EMPTY_COLLECTION;
    private static AtomicReference<Policy.PolicyInfo> policy;
    private static final Debug debug;
    private WeakHashMap<ProtectionDomain.Key, PermissionCollection> pdMapping;
    
    public Policy() {
    }
    
    static boolean isSet() {
        Policy.PolicyInfo policyInfo = (Policy.PolicyInfo)Policy.policy.get();
        return policyInfo.policy != null && policyInfo.initialized;
    }
    
    private static void checkPermission(String str) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new SecurityPermission("createPolicy." + str));
        }
    }
    
    public static Policy getPolicy() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.GET_POLICY_PERMISSION);
        }
        return getPolicyNoCheck();
    }
    
    static Policy getPolicyNoCheck() {
        Policy.PolicyInfo policyInfo = (Policy.PolicyInfo)Policy.policy.get();
        if (!policyInfo.initialized || policyInfo.policy == null) {
            synchronized (Policy.class) {
                Policy.PolicyInfo newValue = (Policy.PolicyInfo)Policy.policy.get();
                if (newValue.policy == null) {
                    final String className = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                        @Override
                        public String run() {
                            return Security.getProperty("policy.provider");
                        }
                    });
                    if (className == null) {
                        className = "sun.security.provider.PolicyFile";
                    }
                    try {
                        newValue = new Policy.PolicyInfo((Policy)Class.forName(className).newInstance(), true);
                    }
                    catch (Exception ex) {
                        PolicyFile o = new PolicyFile();
                        Policy.policy.set(new Policy.PolicyInfo(o, false));
                        Policy policy = (Policy)AccessController.doPrivileged(new PrivilegedAction<Policy>() {
                            @Override
                            public Policy run() {
                                try {
                                    ClassLoader classLoader = ClassLoader.getSystemClassLoader();
                                    ClassLoader loader = null;
                                    while (classLoader != null) {
                                        loader = classLoader;
                                        classLoader = classLoader.getParent();
                                    }
                                    return (loader != null) ? ((Policy)Class.forName(className, true, loader).newInstance()) : null;
                                }
                                catch (Exception ex) {
                                    if (Policy.debug != null) {
                                        Policy.debug.println("policy provider " + className + " not available");
                                        ex.printStackTrace();
                                    }
                                    return null;
                                }
                            }
                        });
                        if (policy != null) {
                            newValue = new Policy.PolicyInfo(policy, true);
                        }
                        else {
                            if (Policy.debug != null) {
                                Policy.debug.println("using sun.security.provider.PolicyFile");
                            }
                            newValue = new Policy.PolicyInfo(o, true);
                        }
                    }
                    Policy.policy.set(newValue);
                }
                return newValue.policy;
            }
        }
        return policyInfo.policy;
    }
    
    public static void setPolicy(Policy policy) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new SecurityPermission("setPolicy"));
        }
        if (policy != null) {
            initPolicy(policy);
        }
        synchronized (Policy.class) {
            Policy.policy.set(new Policy.PolicyInfo(policy, policy != null));
        }
    }
    
    private static void initPolicy(final Policy policy) {
        ProtectionDomain protectionDomain = (ProtectionDomain)AccessController.doPrivileged(new PrivilegedAction<ProtectionDomain>() {
            @Override
            public ProtectionDomain run() {
                return policy.getClass().getProtectionDomain();
            }
        });
        Object permissions = null;
        synchronized (policy) {
            if (policy.pdMapping == null) {
                policy.pdMapping = new WeakHashMap();
            }
        }
        if (protectionDomain.getCodeSource() != null) {
            Policy policy2 = ((Policy.PolicyInfo)Policy.policy.get()).policy;
            if (policy2 != null) {
                permissions = policy2.getPermissions(protectionDomain);
            }
            if (permissions == null) {
                permissions = new Permissions();
                permissions.add(SecurityConstants.ALL_PERMISSION);
            }
            synchronized (policy.pdMapping) {
                policy.pdMapping.put(protectionDomain.key, permissions);
            }
        }
    }
    
    public static Policy getInstance(String algorithm, Policy.Parameters param) throws NoSuchAlgorithmException {
        checkPermission(algorithm);
        try {
            GetInstance.Instance instance = GetInstance.getInstance("Policy", PolicySpi.class, algorithm, param);
            return new Policy.PolicyDelegate((PolicySpi)instance.impl, instance.provider, algorithm, param);
        }
        catch (NoSuchAlgorithmException ex) {
            return handleException(ex);
        }
    }
    
    public static Policy getInstance(String algorithm, Policy.Parameters param, String provider) throws NoSuchProviderException, NoSuchAlgorithmException {
        if (provider == null || provider.length() == 0) {
            throw new IllegalArgumentException("missing provider");
        }
        checkPermission(algorithm);
        try {
            GetInstance.Instance instance = GetInstance.getInstance("Policy", PolicySpi.class, algorithm, param, provider);
            return new Policy.PolicyDelegate((PolicySpi)instance.impl, instance.provider, algorithm, param);
        }
        catch (NoSuchAlgorithmException ex) {
            return handleException(ex);
        }
    }
    
    public static Policy getInstance(String algorithm, Policy.Parameters param, Provider provider) throws NoSuchAlgorithmException {
        if (provider == null) {
            throw new IllegalArgumentException("missing provider");
        }
        checkPermission(algorithm);
        try {
            GetInstance.Instance instance = GetInstance.getInstance("Policy", PolicySpi.class, algorithm, param, provider);
            return new Policy.PolicyDelegate((PolicySpi)instance.impl, instance.provider, algorithm, param);
        }
        catch (NoSuchAlgorithmException ex) {
            return handleException(ex);
        }
    }
    
    private static Policy handleException(NoSuchAlgorithmException ex) throws NoSuchAlgorithmException {
        Throwable cause = ex.getCause();
        if ((cause instanceof IllegalArgumentException)) {
            throw (IllegalArgumentException)cause;
        }
        throw ex;
    }
    
    public Provider getProvider() {
        return null;
    }
    
    public String getType() {
        return null;
    }
    
    public Policy.Parameters getParameters() {
        return null;
    }
    
    public PermissionCollection getPermissions(CodeSource codeSource) {
        return Policy.UNSUPPORTED_EMPTY_COLLECTION;
    }
    
    public PermissionCollection getPermissions(ProtectionDomain protectionDomain) {
        PermissionCollection collection = null;
        if (protectionDomain == null) {
            return new Permissions();
        }
        if (this.pdMapping == null) {
            initPolicy(this);
        }
        synchronized (this.pdMapping) {
            collection = (PermissionCollection)this.pdMapping.get(protectionDomain.key);
        }
        if (collection != null) {
            Permissions permissions2 = new Permissions();
            synchronized (collection) {
                Enumeration<Permission> elements = collection.elements();
                while (elements.hasMoreElements()) {
                    permissions2.add((Permission)elements.nextElement());
                }
            }
            return permissions2;
        }
        PermissionCollection permissions = this.getPermissions(protectionDomain.getCodeSource());
        if (permissions == null || permissions == Policy.UNSUPPORTED_EMPTY_COLLECTION) {
            permissions = new Permissions();
        }
        this.addStaticPerms(permissions, protectionDomain.getPermissions());
        return permissions;
    }
    
    private void addStaticPerms(PermissionCollection collection, PermissionCollection collection2) {
        if (collection2 != null) {
            synchronized (collection2) {
                Enumeration<Permission> elements = collection2.elements();
                while (elements.hasMoreElements()) {
                    collection.add((Permission)elements.nextElement());
                }
            }
        }
    }
    
    public boolean implies(ProtectionDomain protectionDomain, Permission permission) {
        if (this.pdMapping == null) {
            initPolicy(this);
        }
        PermissionCollection collection;
        synchronized (this.pdMapping) {
            collection = (PermissionCollection)this.pdMapping.get(protectionDomain.key);
        }
        if (collection != null) {
            return collection.implies(permission);
        }
        PermissionCollection permissions = this.getPermissions(protectionDomain);
        if (permissions == null) {
            return false;
        }
        synchronized (this.pdMapping) {
            this.pdMapping.put(protectionDomain.key, permissions);
        }
        return permissions.implies(permission);
    }
    
    public void refresh() {
    }
    
    static {
        UNSUPPORTED_EMPTY_COLLECTION = new Policy.UnsupportedEmptyCollection();
        Policy.policy = new AtomicReference(new Policy.PolicyInfo(null, false));
        debug = Debug.getInstance("policy");
    }
    
    private static class PolicyInfo
    {
        final Policy policy;
        final boolean initialized;
        
        PolicyInfo(Policy policy, boolean initialized) {
            this.policy = policy;
            this.initialized = initialized;
        }
    }
    
    private static class PolicyDelegate extends Policy
    {
        private PolicySpi spi;
        private Provider p;
        private String type;
        private Policy.Parameters params;
        
        private PolicyDelegate(PolicySpi spi, Provider p4, String type, Policy.Parameters params) {
            this.spi = spi;
            this.p = p4;
            this.type = type;
            this.params = params;
        }
        
        @Override
        public String getType() {
            return this.type;
        }
        
        @Override
        public Policy.Parameters getParameters() {
            return this.params;
        }
        
        @Override
        public Provider getProvider() {
            return this.p;
        }
        
        @Override
        public PermissionCollection getPermissions(CodeSource codeSource) {
            return this.spi.engineGetPermissions(codeSource);
        }
        
        @Override
        public PermissionCollection getPermissions(ProtectionDomain protectionDomain) {
            return this.spi.engineGetPermissions(protectionDomain);
        }
        
        @Override
        public boolean implies(ProtectionDomain protectionDomain, Permission permission) {
            return this.spi.engineImplies(protectionDomain, permission);
        }
        
        @Override
        public void refresh() {
            this.spi.engineRefresh();
        }
    }
    
    private static class UnsupportedEmptyCollection extends PermissionCollection
    {
        private static final long serialVersionUID = -8492269157353014774L;
        private Permissions perms;
        
        public UnsupportedEmptyCollection() {
            (this.perms = new Permissions()).setReadOnly();
        }
        
        @Override
        public void add(Permission permission) {
            this.perms.add(permission);
        }
        
        @Override
        public boolean implies(Permission permission) {
            return this.perms.implies(permission);
        }
        
        @Override
        public Enumeration<Permission> elements() {
            return this.perms.elements();
        }
    }
    
    public interface Parameters
    {
    }
}
